# Copyright (c) 2022 Samuel J. McKelvie
#
# MIT License - See LICENSE file accompanying this package.
#

"""Package version information"""

# import importlib.metadata as _metadata
# __version = _metadata.version(__package__.replace('_','-')) #  e.g., '0.1.0'


# The following line is automatically updated with "semantic-release version"
__version__ =  "1.0.0"

__all__ = [ '__version__' ]
